import * as fs from 'node:fs';
import { homedir } from 'node:os';
import { dirname, extname, join as join$1 } from 'node:path';
import { d as defineCommand, c as consola, a as colors, r as runCommand } from '../shared/nuxi.bd0a2fa0.mjs';
import 'node:util';
import 'node:process';
import 'node:tty';
import { a as addDependency } from '../shared/nuxi.e773be50.mjs';
import { j as joinURL } from '../shared/nuxi.deb0a220.mjs';
import { g as getProjectPackage, f as fetchModules, a as getNuxtVersion, c as checkNuxtCompatibility, $ as $fetch } from '../shared/nuxi.ea84a606.mjs';
import { s as semver } from '../shared/nuxi.15c58622.mjs';
import { r as resolvePath } from '../shared/nuxi.8054e4da.mjs';
import { S as SUPPORTED_EXTENSIONS } from '../shared/nuxi.c6d79f8e.mjs';
import { mkdir, writeFile, readFile } from 'node:fs/promises';
import 'node:module';
import '../shared/nuxi.96236dcf.mjs';
import { j as join, s as sharedArgs, r as resolve } from '../shared/nuxi.cc8b5ffe.mjs';
import 'node:url';
import '../shared/nuxi.00903207.mjs';
import 'node:http';
import 'node:https';
import 'node:zlib';
import 'node:stream';
import 'node:buffer';
import 'node:net';
import '../shared/nuxi.4ac76f59.mjs';
import '../shared/nuxi.bd137ecf.mjs';
import './satisfies.mjs';
import '../shared/nuxi.2155838d.mjs';
import 'node:assert';
import 'node:v8';
import 'fs';
import 'path';
import 'os';
import 'crypto';
import 'node:crypto';
import 'node:perf_hooks';
import 'node:vm';
import 'assert';
import 'module';
import 'process';
import 'tty';
import 'url';
import 'util';
import 'v8';

const UPDATABLE_EXTS = [".js", ".ts", ".mjs", ".cjs", ".mts", ".cts"];
async function updateConfig(opts) {
  const { parseModule } = await import('./index9.mjs');
  let configFile = await _tryResolve(
    `./${opts.configFile}`,
    opts.cwd,
    SUPPORTED_EXTENSIONS
  ) || await _tryResolve(
    `./.config/${opts.configFile}`,
    opts.cwd,
    SUPPORTED_EXTENSIONS
  ) || await _tryResolve(
    `./.config/${opts.configFile.split(".")[0]}`,
    opts.cwd,
    SUPPORTED_EXTENSIONS
  );
  let created = false;
  if (!configFile) {
    configFile = join(
      opts.cwd,
      opts.configFile + (opts.createExtension || ".ts")
    );
    const createResult = await opts.onCreate?.({ configFile }) ?? true;
    if (!createResult) {
      throw new Error("Config file creation aborted.");
    }
    const content = typeof createResult === "string" ? createResult : `export default {}
`;
    await mkdir(dirname(configFile), { recursive: true });
    await writeFile(configFile, content, "utf8");
    created = true;
  }
  const ext = extname(configFile);
  if (!UPDATABLE_EXTS.includes(ext)) {
    throw new Error(
      `Unsupported config file extension: ${ext} (${configFile}) (supported: ${UPDATABLE_EXTS.join(", ")})`
    );
  }
  const contents = await readFile(configFile, "utf8");
  const _module = parseModule(contents, opts.magicast);
  const defaultExport = _module.exports.default;
  if (!defaultExport) {
    throw new Error("Default export is missing in the config file!");
  }
  const configObj = defaultExport.$type === "function-call" ? defaultExport.$args[0] : defaultExport;
  await opts.onUpdate?.(configObj);
  await writeFile(configFile, _module.generate().code);
  return {
    configFile,
    created
  };
}
function _tryResolve(path, cwd, exts) {
  return resolvePath(path, {
    url: join(cwd, "_index.js"),
    extensions: exts
  }).catch(() => void 0);
}

const add = defineCommand({
  meta: {
    name: "add",
    description: "Add Nuxt modules"
  },
  args: {
    ...sharedArgs,
    moduleName: {
      type: "positional",
      description: "Module name"
    },
    skipInstall: {
      type: "boolean",
      description: "Skip npm install"
    },
    skipConfig: {
      type: "boolean",
      description: "Skip nuxt.config.ts update"
    }
  },
  async setup(ctx) {
    const cwd = resolve(ctx.args.cwd || ".");
    const projectPkg = await getProjectPackage(cwd);
    if (!projectPkg.dependencies?.nuxt && !projectPkg.devDependencies?.nuxt) {
      consola.warn(`No \`nuxt\` dependency detected in \`${cwd}\`.`);
      const shouldContinue = await consola.prompt(
        `Do you want to continue anyway?`,
        {
          type: "confirm",
          initial: false
        }
      );
      if (shouldContinue !== true) {
        return false;
      }
    }
    const r = await resolveModule(ctx.args.moduleName, cwd);
    if (r === false) {
      return;
    }
    if (!ctx.args.skipInstall) {
      const isDev = Boolean(projectPkg.devDependencies?.nuxt);
      consola.info(
        `Installing \`${r.pkg}\`${isDev ? " development" : ""} dependency`
      );
      const res = await addDependency(r.pkg, { cwd, dev: isDev, installPeerDependencies: true }).catch(
        (error) => {
          consola.error(error);
          return consola.prompt(
            `Install failed for ${colors.cyan(
              r.pkg
            )}. Do you want to continue adding the module to ${colors.cyan(
              "nuxt.config"
            )}?`,
            {
              type: "confirm",
              initial: false
            }
          );
        }
      );
      if (res === false) {
        return;
      }
    }
    if (!ctx.args.skipConfig) {
      await updateConfig({
        cwd,
        configFile: "nuxt.config",
        async onCreate() {
          consola.info(`Creating \`nuxt.config.ts\``);
          return getDefaultNuxtConfig();
        },
        async onUpdate(config) {
          if (!config.modules) {
            config.modules = [];
          }
          if (config.modules.includes(r.pkgName)) {
            consola.info(`\`${r.pkgName}\` is already in the \`modules\``);
            return;
          }
          consola.info(`Adding \`${r.pkgName}\` to the \`modules\``);
          config.modules.push(r.pkgName);
        }
      }).catch((error) => {
        consola.error(`Failed to update \`nuxt.config\`: ${error.message}`);
        consola.error(`Please manually add \`${r.pkgName}\` to the \`modules\` in \`nuxt.config.ts\``);
        return null;
      });
    }
    const args = Object.entries(ctx.args).filter(([k]) => k in sharedArgs).map(([k, v]) => `--${k}=${v}`);
    await runCommand("prepare", args);
  }
});
function getDefaultNuxtConfig() {
  return `
// https://nuxt.com/docs/api/configuration/nuxt-config
export default defineNuxtConfig({
  modules: []
})`;
}
const packageRegex = /^(@[a-z0-9-~][a-z0-9-._~]*\/)?([a-z0-9-~][a-z0-9-._~]*)(@[^@]+)?$/;
async function resolveModule(moduleName, cwd) {
  let pkgName = moduleName;
  let pkgVersion;
  const reMatch = moduleName.match(packageRegex);
  if (reMatch) {
    if (reMatch[3]) {
      pkgName = `${reMatch[1] || ""}${reMatch[2] || ""}`;
      pkgVersion = reMatch[3].slice(1);
    }
  } else {
    consola.error(`Invalid package name \`${pkgName}\`.`);
    return false;
  }
  const modulesDB = await fetchModules().catch((err) => {
    consola.warn("Cannot search in the Nuxt Modules database: " + err);
    return [];
  });
  const matchedModule = modulesDB.find(
    (module) => module.name === moduleName || module.npm === pkgName || module.aliases?.includes(pkgName)
  );
  if (matchedModule?.npm) {
    pkgName = matchedModule.npm;
  }
  if (matchedModule && matchedModule.compatibility.nuxt) {
    const nuxtVersion = await getNuxtVersion(cwd);
    if (!checkNuxtCompatibility(matchedModule, nuxtVersion)) {
      consola.warn(
        `The module \`${pkgName}\` is not compatible with Nuxt \`${nuxtVersion}\` (requires \`${matchedModule.compatibility.nuxt}\`)`
      );
      const shouldContinue = await consola.prompt(
        "Do you want to continue installing incompatible version?",
        {
          type: "confirm",
          initial: false
        }
      );
      if (shouldContinue !== true) {
        return false;
      }
    }
    const versionMap = matchedModule.compatibility.versionMap;
    if (versionMap) {
      for (const [_nuxtVersion, _moduleVersion] of Object.entries(versionMap)) {
        if (semver.satisfies(nuxtVersion, _nuxtVersion)) {
          if (!pkgVersion) {
            pkgVersion = _moduleVersion;
          } else {
            consola.warn(
              `Recommended version of \`${pkgName}\` for Nuxt \`${nuxtVersion}\` is \`${_moduleVersion}\` but you have requested \`${pkgVersion}\``
            );
            pkgVersion = await consola.prompt("Choose a version:", {
              type: "select",
              options: [_moduleVersion, pkgVersion]
            });
          }
          break;
        }
      }
    }
  }
  pkgVersion = pkgVersion || "latest";
  const pkgScope = pkgName.startsWith("@") ? pkgName.split("/")[0] : null;
  const registry = await detectNpmRegistry(pkgScope);
  const pkg = await $fetch(joinURL(registry, `${pkgName}/${pkgVersion}`));
  const pkgDependencies = Object.assign(
    pkg.dependencies || {},
    pkg.devDependencies || {}
  );
  if (!pkgDependencies["nuxt"] && !pkgDependencies["nuxt-edge"] && !pkgDependencies["@nuxt/kit"]) {
    consola.warn(`It seems that \`${pkgName}\` is not a Nuxt module.`);
    const shouldContinue = await consola.prompt(
      `Do you want to continue installing \`${pkgName}\` anyway?`,
      {
        type: "confirm",
        initial: false
      }
    );
    if (shouldContinue !== true) {
      return false;
    }
  }
  return {
    nuxtModule: matchedModule,
    pkg: `${pkgName}@${pkgVersion}`,
    pkgName,
    pkgVersion
  };
}
async function detectNpmRegistry(scope) {
  if (process.env.COREPACK_NPM_REGISTRY) {
    return process.env.COREPACK_NPM_REGISTRY;
  }
  const userNpmrcPath = join$1(homedir(), ".npmrc");
  const cwdNpmrcPath = join$1(process.cwd(), ".npmrc");
  const registry = await getRegistryFromFile([cwdNpmrcPath, userNpmrcPath], scope);
  if (registry) {
    process.env.COREPACK_NPM_REGISTRY = registry;
  }
  return registry || "https://registry.npmjs.org";
}
async function getRegistryFromFile(paths, scope) {
  for (const npmrcPath of paths) {
    let fd;
    try {
      fd = await fs.promises.open(npmrcPath, "r");
      if (await fd.stat().then((r) => r.isFile())) {
        const npmrcContent = await fd.readFile("utf-8");
        if (scope) {
          const scopedRegex = new RegExp(`^${scope}:registry=(.+)$`, "m");
          const scopedMatch = npmrcContent.match(scopedRegex);
          if (scopedMatch) {
            return scopedMatch[1].trim();
          }
        }
        const defaultRegex = /^\s*registry=(.+)$/m;
        const defaultMatch = npmrcContent.match(defaultRegex);
        if (defaultMatch) {
          return defaultMatch[1].trim();
        }
      }
    } catch {
    } finally {
      await fd?.close();
    }
  }
  return null;
}

export { add as default };
